/*
* Copyright (c) 2002-2004 by Victor Derevyanko, RAMMuS Group
* www: http://www.rammus.ru
* e-mail: wingspan@yandex.ru
*/
#include "StdAfx.h"
#include "panel.h"
#include "stlcatalogs.h"
#include "lang.h"
#include "Executor.h"
#include "DialogEditCatalog.h"
#include "DialogEditShortcut.h"
#include "DialogMove.h"
#include <algorithm>
#include <ShLwApi.h>
#include "strings_utils.h"

using namespace nf;

namespace nf
{
	class CPanelItemFinder
	{	//      
		tstring m_ItemName;
		ULONG m_ItemsType;
	public:
		CPanelItemFinder(TCHAR const* ItemName, ULONG ItemsType = CPanel::FG_ANY)
			: m_ItemName(ItemName), m_ItemsType(ItemsType)
		{ }
		int operator()()
		{
			PanelInfo pi;
			g_PluginInfo.Control(INVALID_HANDLE_VALUE, FCTL_GETPANELINFO, &pi);
			for (int i = 0; i < pi.ItemsNumber; ++i)
			{
				if (m_ItemName == tstring(pi.PanelItems[i].FindData.cFileName))
				{
					bool bIsCatalog = (pi.PanelItems[i].FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0; 
					if (((m_ItemsType & CPanel::FG_CATALOGS) && bIsCatalog) || 
						((m_ItemsType & CPanel::FG_SHORTCUTS) && (!bIsCatalog)))
					{
						return i;
					}
				}
			}
			return 0;		
		}
	};

	class CPanelUpdater
	{
		ULONG m_nCurrentItem;
		nf::CPanel * const m_pPanel;
		int m_OpMode;
		std::auto_ptr<CPanelItemFinder> m_pif;
	public:
		CPanelUpdater(nf::CPanel * const p
			, ULONG nCurrentItem = 0
			, int OpMode = 0	//   
			) 
			: m_nCurrentItem(nCurrentItem), m_pPanel(p), m_OpMode(OpMode), m_pif(0)
		{ }
		void SetCursorOnItem(tstring const&PanelItemName, ULONG ItemTypes = CPanel::FG_ANY)
		{	//         
			//        
			m_pif = std::auto_ptr<CPanelItemFinder>(new CPanelItemFinder(PanelItemName.c_str(), ItemTypes));
		}
		void Update(ULONG fgUpdate = CPanel::FG_ALL)
		{	//       FAR
			m_pPanel->UpdateListItems(fgUpdate);
			g_PluginInfo.Control(m_pPanel, FCTL_UPDATEPANEL, 0);
			if (m_OpMode & OPM_FIND) return;

			if (m_pif.get()) m_nCurrentItem = (*m_pif)();	//   ,  

			PanelInfo panel_info;
			g_PluginInfo.Control(m_pPanel, FCTL_GETPANELSHORTINFO, &panel_info);

			PanelRedrawInfo pri; 
			pri.CurrentItem = m_nCurrentItem;
			pri.TopPanelItem = (m_nCurrentItem > panel_info.PanelRect.bottom)
				? m_nCurrentItem 
				: 0;
			g_PluginInfo.Control(m_pPanel, FCTL_REDRAWPANEL, &pri);
		}

		void UpdateAnother()
		{	//   FAR
			PanelInfo pi;
			g_PluginInfo.Control(m_pPanel, FCTL_GETANOTHERPANELINFO, &pi);

			PanelRedrawInfo pri; 
			pri.CurrentItem = pi.CurrentItem;
			pri.TopPanelItem = 0;
			g_PluginInfo.Control(m_pPanel, FCTL_UPDATEANOTHERPANEL, 0);
			g_PluginInfo.Control(m_pPanel, FCTL_REDRAWANOTHERPANEL, &pri);
		}
	}; //CPanelUpdater
}

CPanel::CPanel(tstring catalog)
: m_PanelTitle(catalog)
, m_RegNotify(g_pK->m_S.get_NamedFolders_reg_key().c_str())
{
	m_CurrentCatalog = SLASH_CATS + catalog;
	remove_trailing_chars(m_CurrentCatalog, SLASH_CATS_CHAR);

	PanelInfo p;
	g_PluginInfo.Control(INVALID_HANDLE_VALUE, FCTL_GETPANELINFO, &p);
	m_PreviousDir = p.CurDir;

	UpdateListItems();
}

CPanel::~CPanel(void)
{
	tmap_panelitems::iterator p = m_find_cash.begin();
	while (p != m_find_cash.end())
	{
		delete p->second;
	}
}


void CPanel::GetOpenPluginInfo(struct OpenPluginInfo *pi)
{
	pi->StructSize = sizeof(*pi);
	pi->Flags = OPIF_USEFILTER 
		| OPIF_ADDDOTS 
		| OPIF_SHOWNAMESONLY 
		| OPIF_SHOWRIGHTALIGNNAMES
		| OPIF_RAWSELECTION
		| OPIF_SHOWPRESERVECASE
		| OPIF_USEHIGHLIGHTING;
	pi->HostFile = 0;
	pi->CurDir = const_cast<char*>(m_CurrentCatalog.c_str());
	pi->Format = (char*)nf::GetMsg(lg::NAMEDFOLDERS);
	m_PanelTitle = get_panel_title().c_str();
	pi->PanelTitle = const_cast<char*>(m_PanelTitle.c_str());

	pi->InfoLines = 0;
	pi->InfoLinesNumber = 0;

	pi->DescrFiles = 0;
	pi->DescrFilesNumber = 0;

	int panel_mode_id = g_pK->m_S.GetValue(nf::ST_PANEL_MODE);	
	if ((panel_mode_id > 10) || (panel_mode_id < 0)) 
		panel_mode_id = 6;

	const int NUM_PANEL_MODES = 10;
	static struct PanelMode PanelModesArray[NUM_PANEL_MODES];
	PanelMode& panel_mode = PanelModesArray[panel_mode_id];
	panel_mode.ColumnTypes = NULL;
	panel_mode.ColumnWidths = "8,0";
	panel_mode.FullScreen = FALSE;
	panel_mode.Reserved[0] = 0;
	panel_mode.Reserved[1] = 0;

	pi->PanelModesArray = PanelModesArray;
	pi->PanelModesNumber = NUM_PANEL_MODES;
	pi->StartPanelMode='6';

	static struct KeyBarTitles KeyBar;
	memset(&KeyBar, 0, sizeof(KeyBar));
	KeyBar.Titles[3-1] = "";
	KeyBar.Titles[5-1] = (char*)nf::GetMsg(lg::F5);
	KeyBar.Titles[6-1] = (char*)nf::GetMsg(lg::F6);
	KeyBar.Titles[8-1] = (char*)nf::GetMsg(lg::F8);
	KeyBar.Titles[7-1] = (char*)nf::GetMsg(lg::F7);
	pi->KeyBar = &KeyBar;
}


int CPanel::SetDirectory(const char *Dir, int OpMode)
{	//   -   ,    
	//  
	CPanelUpdater pu(this, 0, OpMode);

	if (tstring(_T("..")) == Dir)
	{
		return go_to_up_folder(OpMode);
	} else {
		pu.SetCursorOnItem(m_CurrentCatalog, nf::CPanel::FG_CATALOGS);
		m_CurrentCatalog = m_CurrentCatalog + tstring(SLASH_CATS) + tstring(Dir);
	}

	pu.Update();
	return TRUE;
}


bool CPanel::IsSelectedItemIsCatalog(PanelInfo &pi, int nSelectedItem) const
{
	return (pi.SelectedItems[nSelectedItem].FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY);
}

inline kr::tshortcut_info& CPanel::GetSelectedShortcut(PanelInfo &pi
													   , kr::tshortcut_info& sh
													   , int nSelectedItem) 
{
	sh.catalog = m_CurrentCatalog;
	sh.shortcut = pi.SelectedItems[nSelectedItem].FindData.cFileName;
	sh.bIsTemporary = 0 != pi.SelectedItems[nSelectedItem].CRC32;
	return sh;
}

inline tstring CPanel::GetSelectedCatalog(PanelInfo &pi, int nSelectedItem) 
{
	return m_CurrentCatalog + tstring(_T("/")) + pi.SelectedItems[nSelectedItem].FindData.cFileName;
}

int CPanel::ProcessKey(int Key, unsigned int ControlState)
{	//    FAR 
	PanelInfo pi;
	kr::tshortcut_info sh;

	switch (Key)
	{
	case VK_RETURN:
	case VK_LBUTTON: 
		if (! g_PluginInfo.Control(this, FCTL_GETPANELINFO, &pi)) return FALSE;

		if (PKF_SHIFT == ControlState)
		{	//  
			if (! IsSelectedItemIsCatalog(pi)) 
				nf::open_shortcut_in_explorer(this, GetSelectedShortcut(pi, sh), tstring());
			return TRUE;	
		}
		if (! pi.SelectedItemsNumber) return FALSE;	// ".."

		//     /  
		if (! IsSelectedItemIsCatalog(pi))	
		{	//   
			if (PKF_CONTROL != ControlState)
			{
				nf::open_shortcut(this, GetSelectedShortcut(pi, sh), tstring());
				return TRUE;	//itsn't necessary to update active panel
			}
			else {//  
				nf::open_shortcut(this, GetSelectedShortcut(pi, sh), tstring(), false);
				return TRUE;	//itsn't necessary to update active panel
			}
		} else return FALSE;	
		break;
	case VK_PRIOR:	//Ctrp + PgUp = BACKSPACE
		if (PKF_CONTROL == ControlState) return go_to_up_folder(0);
		return FALSE;
	case VK_F8: 
		// ()/()
		if (g_PluginInfo.Control(this, FCTL_GETPANELINFO, &pi))
		{
			if (! pi.SelectedItemsNumber) return TRUE;	// ".."

			CPanelUpdater pu(this, pi.CurrentItem);
			ULONG count = (pi.SelectedItemsNumber > 1) ? pi.SelectedItemsNumber : 1;
			bool bRet = false;
			for (int i = 0; i < pi.SelectedItemsNumber; ++i)
				bRet |= delete_item(pi, i); 
			if (bRet) pu.Update();
		} 
		return TRUE;
	case 'R':
		//  
		break;
	case VK_F5:	
		if (g_PluginInfo.Control(this, FCTL_GETPANELINFO, &pi)) 
		{
			if (! pi.SelectedItemsNumber) return TRUE;	// ".."
			CopyItems(pi);
		}
		return TRUE;
	case VK_F6: 
		if (g_PluginInfo.Control(this, FCTL_GETPANELINFO, &pi)) 
		{
			if (! pi.SelectedItemsNumber) return TRUE;	// ".."
			MoveItems(pi, false);
		}
		return TRUE;
	case VK_F4:
		if (g_PluginInfo.Control(this, FCTL_GETPANELINFO, &pi)) 
		{
			if (PKF_SHIFT == ControlState) InsertShortcut(pi);
			else 
			if (! IsSelectedItemIsCatalog(pi)) EditShortcut(GetSelectedShortcut(pi, sh), pi);
		} 
		return TRUE;
	case VK_F3: 
		return TRUE;	//   FAR'
	default:
		return FALSE;
	}; //switch

	CPanelUpdater pu(this);
	pu.Update();
	
	return TRUE;
}

int CPanel::MakeDirectory (char *Name, int OpMode)
{	//  
	CPanelUpdater pu(this);

	PanelInfo pi;
	kr::tcatalog_info c;

	if (g_PluginInfo.Control(this, FCTL_GETPANELINFO, &pi))
	if (! CreateCatalog(m_CurrentCatalog, c))
	{
		return -1;	// 
	}

	pu.Update(FG_CATALOGS);
	return TRUE;
}

int CPanel::PutFiles(struct PluginPanelItem *PanelItem
					 , int ItemsNumber
					 , int Move
					 , int OpMode)
{	//    -     ,   
	//! xml-       
	if (OpMode & OPM_SILENT) return FALSE;

	for (int i = 0; i < ItemsNumber; ++i)
	{
		PanelInfo p;
		g_PluginInfo.Control(this, FCTL_GETANOTHERPANELINFO, &p);
		
		if (PanelItem[i].FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)	
		{
			kr::tshortcut_info sht;
			sht.bIsTemporary = false;
			sht.catalog = m_CurrentCatalog;
			sht.shortcut = PanelItem[i].FindData.cFileName; 
			tstring value = p.CurDir;
			value += _T("\\");
			value += PanelItem[i].FindData.cFileName; 
			nf::add_shortcut(this
				, sht
				, value.c_str()
				, true //implicit
			);
		};
	}

	CPanelUpdater pu(this, 0, OpMode);
	pu.Update();

	return TRUE;
}

int CPanel::GetFindData(PluginPanelItem **pPanelItem, int *pItemsNumber, int OpMode)
{	
	static struct PanelInfo PInfo;
	if (OpMode & OPM_FIND)
	{	//     (..   
		//     )
		tpanelitems *pm = new tpanelitems();
		*pm = m_PanelItems;
		m_find_cash.insert(std::make_pair(&(*pm)[0], pm));
		*pPanelItem = &(*pm)[0];
		*pItemsNumber = static_cast<int>(pm->size());
	} else {
		*pPanelItem = &m_PanelItems[0];
		*pItemsNumber = static_cast<int>(m_PanelItems.size());
	}
	return TRUE;
}

void CPanel::FreeFindData(struct PluginPanelItem *PanelItem, int ItemsNumber)
{	
	tmap_panelitems::iterator p = m_find_cash.find(PanelItem);
	if (p != m_find_cash.end())
	{
		tpanelitems *pm = p->second;
		m_find_cash.erase(p);
		delete pm;
	}
}

int CPanel::ProcessEvent(int Event, void *Param)
{
	if (FE_REDRAW == Event)
	{ 
		
	} else if (FE_IDLE == Event) 
	{
		if (m_RegNotify.check())
		{
			//,     ... ???
			PanelInfo pi;
			g_PluginInfo.Control(this, FCTL_GETANOTHERPANELSHORTINFO, &pi);
			if (pi.Plugin)
			{
				CPanelUpdater pu(this, pi.CurrentItem);
				pu.Update();
			}

			m_RegNotify.start();
		};
	}
	return FALSE;
}

// -----
BOOL CPanel::MoveItems(PanelInfo &pi
					   , bool bCopy)
{
	bool bSeveralCatalogs = 1 != pi.SelectedItemsNumber;
	bool bFirstItemIsCatalog = IsSelectedItemIsCatalog(pi, 0);
	tstring name;
	if (bFirstItemIsCatalog)
	{
		name = GetSelectedCatalog(pi, 0);
	} else {
		kr::tshortcut_info sh;
		GetSelectedShortcut(pi, sh, 0);
		name = sh.catalog + tstring(SLASH_CATS) + sh.shortcut;			
	};

// ? ,       ...
	tstring default_path = get_inactive_panel_current_catalog();
	bool bPluginOpenedOnInactivePanel = ! default_path.empty();
	
	nf::CDialogMove dlg(name.c_str()
		, default_path
		, bFirstItemIsCatalog
		, bSeveralCatalogs
		, bCopy);
	if (dlg.ShowModal())
	{
		CPanelUpdater pu(this, pi.CurrentItem);
		bool bSingleItem = pi.SelectedItemsNumber == 1;
		
		bool bRet = false;
		for (int i = 0; i < pi.SelectedItemsNumber; ++i)
		{
			tstring new_path = dlg.GetItemName();
			
			if (IsSelectedItemIsCatalog(pi, i))
			{
				tstring new_catalog_name;
				tstring catalog_name = GetSelectedCatalog(pi, i); 
				if (bCopy)
				{
					bRet |= g_pK->copy_catalog(catalog_name.c_str(), new_path);
				} else {
					bRet |= g_pK->move_catalog(catalog_name.c_str(), new_path);
				}
				if (bSingleItem)
				{
			//          , 
			//    .
					if (catalog_name == new_path) pu.SetCursorOnItem(new_catalog_name, FG_CATALOGS);
					//!!!!    ,   
				}
			} else {
				kr::tshortcut_info new_sh;
				kr::tshortcut_info sh;
				GetSelectedShortcut(pi, sh, i);
				if (bCopy)
				{
					bRet |= g_pK->copy_shortcut(sh, new_path, new_sh);
				} else {
					bRet |= g_pK->move_shortcut(sh, new_path, new_sh);
				}
				if (bSingleItem)
				{
			//          , 
			//    .
					if (sh.catalog == new_sh.catalog) pu.SetCursorOnItem(new_sh.shortcut, FG_SHORTCUTS);
				}
			}
		}

		pu.Update();
		pu.UpdateAnother();

		return static_cast<BOOL>(bRet);
	}

	return FALSE;
}

bool CPanel::EditShortcut(kr::tshortcut_info const &shortcut_name,
						  PanelInfo &pi)
{
	kr::tshortcut_info sh;
	get_selected_shortcut_info(sh);
	tstring value;
	g_pK->get_value_for_shortcut(sh, value);
	nf::tshortcut_value_parsed apv;
	nf::tshortcut_value_parsed inpv;
	nf::decode_values(value, apv, inpv);

	nf::CDialogEditShortcut dlg(sh, apv.value, inpv.value, false);
	if (dlg.ShowModal())
	{
		CPanelUpdater pu(this, pi.CurrentItem);
		kr::tshortcut_info const& sh2 = dlg.GetShortcutInfo();
		tstring value2 = nf::encode_values(dlg.GetValueForActivePanel().c_str(), dlg.GetValueForInActivePanel().c_str());
		if (g_pK->modify_shortcut(sh, sh2, &value2))
		{
			if (sh.catalog == sh2.catalog)
				pu.SetCursorOnItem(sh2.shortcut, FG_SHORTCUTS);
			pu.Update();
			return true;
		}
	}
	return false;
}

bool CPanel::InsertShortcut(PanelInfo &pi)
{
	kr::tshortcut_info default_sh;
	nf::tshortcut_value_parsed apv;
	nf::get_implicit_name_and_value(this, false, default_sh.shortcut, apv.value);
	default_sh.bIsTemporary = false;
	default_sh.catalog = m_CurrentCatalog;

	nf::CDialogEditShortcut dlg(default_sh, apv.value, tstring(), true);
	if (dlg.ShowModal())
	{
		CPanelUpdater pu(this, pi.CurrentItem);
		kr::tshortcut_info const& sh = dlg.GetShortcutInfo();
		tstring value = nf::encode_values(dlg.GetValueForActivePanel().c_str(), dlg.GetValueForInActivePanel().c_str());
		if (g_pK->insert_shortcut(sh, value, false))
		{
			pu.SetCursorOnItem(sh.shortcut, FG_SHORTCUTS);
			pu.Update();
			return true;
		}
	}
	return false;
}


bool CPanel::CreateCatalog(tstring const& ParentCatalog, tstring &CreatedCatalogName)
{
	nf::CDialogEditCatalog dlg(ParentCatalog.c_str(), _T(""), true);
	if (dlg.ShowModal())
	{
		CreatedCatalogName = dlg.GetCatalogName(true);
		return nf::add_catalog(this, CreatedCatalogName);
	}
	return false;
}

// ----- private
inline void SetPanelItem(PluginPanelItem& item
						 , char const* name
						 , char const* folder
						 , bool bIsTemporary
						 , bool bIsHidden
						 , bool bIsDirectory)
{
	memset(&item, 0, sizeof(item));
	item.Description = const_cast<char*>(folder);
	item.Flags = 0;
	lstrcpy(item.FindData.cFileName, name);
	item.FindData.dwFileAttributes =  (bIsHidden) ? FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM : 0;
	if (bIsDirectory)
		item.FindData.dwFileAttributes |= FILE_ATTRIBUTE_DIRECTORY;
	item.Owner = NULL;
	item.CustomColumnData = NULL;
	item.CRC32 = bIsTemporary ? 1 : 0;
}

void CPanel::UpdateListItems(DWORD flags)
{
	m_PanelItems.clear();

	sc::CCatalog c(m_CurrentCatalog);
	sc::subcatalogs_sequence cseq = c.get_subcatalogs_sequence();
	sc::shortcuts_sequence tseq = c.get_shortcuts_sequence(true);
	sc::shortcuts_sequence shseq = c.get_shortcuts_sequence(false);

	//  
	if (flags & FG_CATALOGS)
	{
		m_ListCatalogsNames.clear();
		m_ListCatalogsNames.reserve(cseq.size());
		sc::subcatalogs_sequence::const_iterator pc = cseq.begin();
		while (pc != cseq.end())
		{
			sc::catalogs_sequence_item csi(*pc);
			m_ListCatalogsNames.push_back(csi.get_catalog_name());
			++pc;
		}
	}

	//  
	if (flags & FG_SHORTCUTS)
	{
		m_ListShortcutsNames.clear();
		m_ListShortcutsValues.clear();

		m_ListShortcutsNames.reserve(tseq.size() + shseq.size());
		m_ListShortcutsValues.reserve(m_ListShortcutsNames.size());
		//
		sc::shortcuts_sequence::const_iterator pt = tseq.begin();
		while (pt != tseq.end())
		{
			sc::shortcuts_sequence_item si(*pt);
			m_ListShortcutsNames.push_back(si.get_shortcut_name());
			m_ListShortcutsValues.push_back(si.get_value());
			++pt;
		}
		NumberTemporaryShortcuts = static_cast<int>(tseq.size());
		//
		sc::shortcuts_sequence::const_iterator psh = shseq.begin();
		while (psh != shseq.end())
		{
			sc::shortcuts_sequence_item si(*psh);
			m_ListShortcutsNames.push_back(si.get_shortcut_name());
			m_ListShortcutsValues.push_back(si.get_value());
			++psh;
		}
	}

	BOOL bTemporaryAsHidden = g_pK->m_S.GetValue(nf::ST_SHOW_TEMPORARY_AS_HIDDEN);
	m_PanelItems.resize(m_ListCatalogsNames.size() + m_ListShortcutsNames.size());

	for (unsigned int i = 0; i < m_ListCatalogsNames.size(); ++i)
		SetPanelItem(m_PanelItems[i]
			, m_ListCatalogsNames[i].c_str()
			, _T("")
			, false
			, false
			, true
		);

	for (unsigned int i = 0; i < m_ListShortcutsNames.size(); ++i)
		SetPanelItem(m_PanelItems[m_ListCatalogsNames.size() + i]
			, m_ListShortcutsNames[i].c_str()
			, m_ListShortcutsValues[i].c_str()
			, i < NumberTemporaryShortcuts
			, (i < NumberTemporaryShortcuts) && bTemporaryAsHidden
			, false
		);

}

int CPanel::go_to_up_folder(int OpMode)
{
	if (m_CurrentCatalog.empty())
	{
		if (OpMode & OPM_FIND) return FALSE;
		
		g_PluginInfo.Control(this,  FCTL_CLOSEPLUGIN,  (void*)(m_PreviousDir.c_str()));
		return TRUE;
	}
	else 
	{ 
		tstring CurrentCatalogName;
		int nCurrentItem = -1;

		size_t npos = m_CurrentCatalog.find_last_of(SLASH_CATS_CHAR);
		if (npos == tstring::npos)
		{
			m_CurrentCatalog = _T("");
			nCurrentItem = 0;
		} else 
		{	//       
			CurrentCatalogName.assign(m_CurrentCatalog.begin()+npos+1, m_CurrentCatalog.end()); 
			m_CurrentCatalog.erase(npos, m_CurrentCatalog.size()-npos);
		}

		nf::CPanelUpdater pu(this, 0, OpMode);
		pu.SetCursorOnItem(CurrentCatalogName, nf::CPanel::FG_CATALOGS);
		pu.Update();

		return TRUE;
	}

}

void CPanel::get_selected_shortcut_info(kr::tshortcut_info &sh)
{
	PanelInfo pi;
	g_PluginInfo.Control(INVALID_HANDLE_VALUE, FCTL_GETPANELINFO, &pi);
	assert(! IsSelectedItemIsCatalog(pi));
	
	sh.bIsTemporary = pi.SelectedItems[0].FindData.dwFileAttributes && FILE_ATTRIBUTE_TEMPORARY;
	sh.catalog = m_CurrentCatalog;
	sh.shortcut = pi.SelectedItems[0].FindData.cFileName;
}

bool CPanel::delete_item(PanelInfo &pi, int nSelectedItem)
{
	if (IsSelectedItemIsCatalog(pi, nSelectedItem)) 
		return nf::delete_catalog(this, GetSelectedCatalog(pi, nSelectedItem));
	else 
	{
		kr::tshortcut_info sh;
		return nf::delete_shortcut(this, GetSelectedShortcut(pi, sh, nSelectedItem), false);
	}
}

tstring CPanel::get_inactive_panel_current_catalog()
{
	PanelInfo pi;
	g_PluginInfo.Control(INVALID_HANDLE_VALUE, FCTL_GETANOTHERPANELINFO, &pi);
	tstring s = pi.CurDir;
	if (s.find_first_of(_T(":")) != tstring::npos) return _T("");
	else return pi.CurDir;
}

