/*
* Far Named Folders 2.x
* Copyright (c) 2002-2004 by Victor Derevyanko, RAMMuS Group
* www: http://www.rammus.ru
* e-mail: wingspan@yandex.ru
*/
#include "StdAfx.h"
#include "select_variants.h"
#include "stlcatalogs.h"
#include <algorithm>
#include "Parser.h"
#include <functional>
#include <cassert>
#include "codec_for_values.h"
#include "strings_utils.h"

namespace
{
	class check_shortcut 
		: public std::unary_function<sc::shortcuts_sequence_item, void>
	{	//   for_each   select_variants_shortcuts	
		kr::tshortcuts_list& m_list;
		tstring const& m_catalog;
		tstring const& m_pattern;
		bool m_bTemporary;
	public:
		check_shortcut(kr::tshortcuts_list& list
			, tstring const& c
			, tstring const& p
			, bool bTemporary) 
			: m_list(list), m_bTemporary(bTemporary), m_catalog(c), m_pattern(p){}
			void operator()(sc::shortcuts_sequence_item sv)
			{	// ,  ,     list
				tstring name = sv.get_shortcut_name();
				if (parser::token_is_matched_to_pattern(name, m_pattern))
				{
					kr::tshortcut_info info;
					info.bIsTemporary = m_bTemporary;
					info.catalog = m_catalog;
					info.shortcut = name;
					m_list.push_back(info);
				}
			}
	};

	class check_shortcut_values 
		: public std::unary_function<sc::shortcuts_sequence_item, void>
	{	//   for_each   select_variants_shortcuts	
		kr::tshortcuts_list& m_list;
		tstring const& m_catalog;
		tstring m_ValuePattern;
		bool m_bExactCoincidence;
		bool m_bTemporary;
	public:
		check_shortcut_values(kr::tshortcuts_list& list
			, tstring const& c
			, tstring const& value
			, bool bExactCoincidence
			, bool bTemporary) 
			: m_list(list)
			, m_bTemporary(bTemporary)
			, m_catalog(c)
			, m_bExactCoincidence(bExactCoincidence)
		{
			m_ValuePattern = _T("*") + value + _T("*");
		}

		inline bool IsMatch(tstring const &value)
		{
			if (m_bExactCoincidence)
			{
				return parser::token_is_matched_to_pattern(value, m_ValuePattern, false);
				//return value == m_ValuePattern;
			} else 
			{
				return parser::token_is_matched_to_pattern(value, m_ValuePattern, false);
			}
		}

		void operator()(sc::shortcuts_sequence_item sv)
		{	// ,  ,     list
			nf::tshortcut_value_parsed a;
			nf::tshortcut_value_parsed b;
			tstring value = sv.get_value();
			nf::decode_values(value, a, b);
			
			bool bFound = false;

			if (a.bValueEnabled) 
				bFound = IsMatch(a.value);
			else if (b.bValueEnabled) 
				bFound = IsMatch(b.value);

			if (bFound)
			{
				kr::tshortcut_info info;
				info.bIsTemporary = m_bTemporary;
				info.catalog = m_catalog;
				info.shortcut = sv.get_shortcut_name();
				m_list.push_back(info);
			}
		}
	};

	class pattern_extractor
	{	//    
		typedef stlsoft::string_tokeniser_t::const_iterator const_iterator;
		const_iterator m_i;
		const_iterator m_end;
	public:
		pattern_extractor(const_iterator &i, const_iterator end) : m_i(i), m_end(end){}
		pattern_extractor(pattern_extractor& c) : m_i(c.m_i), m_end(c.m_end){}
		tstring find_next() {	
			assert(m_i != m_end);
			return (*m_i++);
		}
		bool eof() {return m_i == m_end;}
	};


	void search_catalogs(sc::CCatalog& c, pattern_extractor& pex, kr::tcatalogs_list& list)
	{
		pattern_extractor pe(pex);
		tstring smask = pe.find_next();
//		if (pe.eof()) smask += _T("*");

		sc::subcatalogs_sequence f(c.get_subcatalogs_sequence());
		sc::subcatalogs_sequence::const_iterator p = f.begin();
		while (p != f.end())
		{
			sc::catalogs_sequence_item cv(*p);
			++p;
			if (! parser::token_is_matched_to_pattern(cv.get_catalog_name().c_str(), smask.c_str())) continue;

		//          
			sc::CCatalog subc(cv, &c);
			if (pe.eof()) {
				tstring s = subc.catalog_path();
				list.push_back(subc.catalog_path()); //     -     
			} else 
				search_catalogs(subc, pe, list); //    
		}
	}

	void search_dirs(tstring current_dir, pattern_extractor &src_pe, kr::tdirs_list& list)
	{	//        
		pattern_extractor pe = src_pe;
		tstring smask = pe.find_next();
		if (pe.eof()) smask += _T("*");

		typedef winstl::findfile_sequence_t tcontainer;
		tcontainer f(current_dir.c_str(), smask.c_str(), tcontainer::directories);
		tcontainer::const_iterator p = f.begin();
		while (p != f.end())
		{
			//          
			tstring dir = *p;
			if (pe.eof()) 
				list.push_back(dir); //     -     
			else 
				search_dirs(dir, pe, list); //    
			++p;
		}
	}

	class shortcuts_sort_perdicate 
		: public std::binary_function<kr::tshortcut_info, kr::tshortcut_info, bool>
	{	//         
		tstring const& m_catalog;
		tstring const& m_shortcut;
	public:
		shortcuts_sort_perdicate(tstring const &catalog, tstring const &shortcut)
			: m_catalog(catalog), m_shortcut(shortcut)
		{}
		int get_hash(kr::tshortcut_info const& s1) 
		{
			return
				! lstrcmpi(s1.shortcut.c_str(), m_shortcut.c_str())
				? 0
				: 1;
		}
		bool operator()(kr::tshortcut_info const & s1, kr::tshortcut_info const &s2)
		{	
			int h1 = get_hash(s1);
			int h2 = get_hash(s2);
			if ((! h1) || (! h2)) 
				return h1 < h2;
			else 
				return s1.shortcut < s2.shortcut;
		}
	};
}


//--- public 
int select_variants_shortcuts(tstring const &catalog, 
							  tstring const &shortcut_pattern, 							  
							  kr::tshortcuts_list& list,
							  bool bSearchInSubcatalogs)
{
	//  x -     x
	//      
	// -   (    )
	sc::CCatalog c(catalog, 0, false);

	{	//   
		::check_shortcut fss(list, catalog, shortcut_pattern, false);		
		sc::shortcuts_sequence seq = c.get_shortcuts_sequence(false);
		std::for_each(seq.begin(), seq.end(), fss);
	}

	{	//   
		::check_shortcut fst(list, catalog, shortcut_pattern, true);		
		sc::shortcuts_sequence seq2 = c.get_shortcuts_sequence(true);
		std::for_each(seq2.begin(), seq2.end(), fst);
	}

	if (bSearchInSubcatalogs) 
	{	//   
		sc::subcatalogs_sequence seqc = c.get_subcatalogs_sequence();
		sc::subcatalogs_sequence::const_iterator p = seqc.begin();
		while (p != seqc.end())
		{
			sc::catalogs_sequence_item cv(*p);
			tstring cat2 = catalog + tstring(SLASH_CATS) + cv.get_catalog_name();
			select_variants_shortcuts(cat2.c_str(), shortcut_pattern, list);
			++p;
		}
	}

	return 0;
}

int select_variants_catalogs(tstring const& pattern	//
							 , kr::tcatalogs_list& list)
{ //     ,     	
  // -   (    )
	sc::CCatalog c;
	stlsoft::string_tokeniser_t t(pattern, SLASH_CATS);
	pattern_extractor p(t.begin(), t.end());
	::search_catalogs(c, p, list);

	return 0;
}

int select_variants_dirs(TCHAR const* root_dir		// 
						 , tstring const& pattern	//
						 , kr::tdirs_list& list)
{ //     ,    					
  // -   (    )
	tstring delimeter = SLASH_DIRS;
	stlsoft::string_tokeniser_t t(pattern, SLASH_DIRS);
	pattern_extractor p(t.begin(), t.end());
	::search_dirs(root_dir, p, list);

	return 0;
}


int sort_by_relevance(kr::tshortcuts_list& list, 
					  tstring const& catalog, 
					  tstring const& shortcut_pattern)
{
	::shortcuts_sort_perdicate eq(catalog, shortcut_pattern);

	//!todo 
	std::sort(list.begin(), list.end(), eq); //remove all exact coincidences to the begining of the list

	kr::tshortcuts_list::iterator plast = list.begin();
	while (plast != list.end()) 
		if (eq.get_hash(*plast) == 0) ++plast;
		else break;

	if (plast == list.end()) 
		return 0;
	else
		return static_cast<int>(plast - list.begin());
}

int select_variants_shortcuts_by_path(tstring catalog
									  , tstring const &value_pattern
									  , kr::tshortcuts_list& list
									  , bool bExactCoincidence
									  , bool bSearchInSubcatalogs)
{
	remove_trailing_chars(catalog, SLASH_CATS_CHAR);
	sc::CCatalog c(catalog, false);

	{	//   
		::check_shortcut_values fss(list, catalog, value_pattern, bExactCoincidence, false);		
		sc::shortcuts_sequence seq = c.get_shortcuts_sequence(false);
		std::for_each(seq.begin(), seq.end(), fss);
	}

	{	//   
		::check_shortcut_values fst(list, catalog, value_pattern, bExactCoincidence, true);		
		sc::shortcuts_sequence seq2 = c.get_shortcuts_sequence(true);
		std::for_each(seq2.begin(), seq2.end(), fst);
	}

	if (bSearchInSubcatalogs) 
	{	//   
		sc::subcatalogs_sequence seqc = c.get_subcatalogs_sequence();
		sc::subcatalogs_sequence::const_iterator p = seqc.begin();
		while (p != seqc.end())
		{
			sc::catalogs_sequence_item cv(*p);
			tstring cat2 = catalog + tstring(SLASH_CATS) + cv.get_catalog_name();
			select_variants_shortcuts_by_path(cat2, value_pattern, list
				, bExactCoincidence, bSearchInSubcatalogs);
			++p;
		}
	}

	return 0;
}
