/*
* Far Named Folders 2.x
* Copyright (c) 2002-2004 by Victor Derevyanko, RAMMuS Group
* www: http://www.rammus.ru
* e-mail: wingspan@yandex.ru
*/
#pragma once
#include "stlsoft.h"
#include <string>
#include <vector>
#include <algorithm>
#include <functional>

#include "Kernel.h"
#include "Registry.h"

//     stl

namespace sc
{
	typedef tstring string_type;
	typedef winstl::reg_value_sequence_t shortcuts_sequence;
	typedef winstl::reg_key_sequence_t subcatalogs_sequence;

class CCatalog;

class shortcuts_sequence_item
{
	typedef winstl::reg_value_t basic_class;
	basic_class m_value;
public:
	shortcuts_sequence_item(){}

	shortcuts_sequence_item(basic_class const &value) : m_value(value){}
	shortcuts_sequence_item(shortcuts_sequence_item const& s) : m_value(s.m_value) {}

	shortcuts_sequence_item& operator=(basic_class const&value)	{m_value = value;}
	string_type get_value() const {	return m_value.value_sz().c_str();}
	string_type get_shortcut_name() const {	return m_value.name().c_str();}
};

class catalogs_sequence_item
{
	typedef winstl::reg_key_t basic_class;
	basic_class m_value;
public:
	catalogs_sequence_item(){}

	catalogs_sequence_item(basic_class const &value) : m_value(value){}
	catalogs_sequence_item(catalogs_sequence_item const& s) : m_value(s.m_value) {}

	catalogs_sequence_item & operator=(basic_class const&value)	{m_value = value;}
	string_type get_catalog_name() const {	return m_value.name().c_str();}

	friend CCatalog;
};

class CCatalog 
{
	typedef winstl::reg_key_t basic_class;
	typedef enum tregs_enum	{REG_STATIC_KEYS, REG_TEMP_KEYS, REG_SUB_CATALOGS, REG_B_SUB_CATALOGS_B};

	basic_class m_key;
	tstring m_CatalogPath;
public:
//   -  
	CCatalog() 
		: m_key(HKEY_CURRENT_USER, get_far_reg_key()) 
	{}

//     
	CCatalog(string_type SubCatalog
		, CCatalog const *pParent = 0
		, bool bCreateIfNotExists = true)
	{
		m_CatalogPath = get_combined_path(SubCatalog.c_str(), pParent);
		tstring regkey = catalog_regkey();
		if (bCreateIfNotExists) 
		{
			CRegistry(HKEY_CURRENT_USER, regkey.c_str());
		}
		m_key = basic_class(HKEY_CURRENT_USER, regkey, KEY_ALL_ACCESS);		
	}

// 
	CCatalog(CCatalog const &catalog) 
		: m_key(catalog.m_key), m_CatalogPath(catalog.m_CatalogPath) 
	{}
//       
	CCatalog(sc::catalogs_sequence_item &c, CCatalog const *pParent) 
	{
		m_CatalogPath = get_combined_path(c.get_catalog_name().c_str(), pParent);
		m_key = basic_class(HKEY_CURRENT_USER, catalog_regkey(), KEY_ALL_ACCESS);		
	}

//
public:
	TCHAR const* catalog_name() const {return m_key.name().c_str();}	// 
	TCHAR const* catalog_path() const //     	
	{
		return m_CatalogPath.c_str();
	}	
	tstring catalog_regkey() const
	{	//    
		//:    "/"          
		//: ( (  )   "/"),   "/"  "catalogs\",      
		tstring key = m_CatalogPath;
		if ((key != SLASH_CATS) && (! key.empty()))
		{
			if (key[0] != SLASH_CATS_CHAR) key = SLASH_CATS_CHAR + key;

			// "/"  "catalogs\"
			size_t pos = 0;
			while (true)
			{
				//   '/'
				pos = key.find(SLASH_CATS, pos);
				if (tstring::npos == pos) break;
				//    'catalogs\'
				key.replace(pos, 1, GetKeyName(REG_B_SUB_CATALOGS_B));
			}
			
			if (key.size() > 1) 
				if (*key.begin() == _T('\\'))
					key.erase(key.begin());	//  /
			return get_far_reg_key() +  _T("\\") + key;
		}
		return get_far_reg_key();
	}
	size_t number_subcatalogs() const
	{
		winstl::reg_key_t c(m_key.get_key_handle(), GetKeyName(REG_SUB_CATALOGS));
		winstl::reg_key_sequence_t s(c);
		return s.size();
	}
	size_t number_shortcuts() const
	{
		winstl::reg_key_t c(m_key.get_key_handle(), GetKeyName(REG_STATIC_KEYS));
		winstl::reg_value_sequence_t s(c);
		return s.size();
	}
	size_t number_temporary_shortcuts() const
	{
		winstl::reg_key_t c(m_key.get_key_handle(), GetKeyName(REG_TEMP_KEYS));
		winstl::reg_value_sequence_t s(c);
		return s.size();
	}
//  
public:
	basic_class get_shortcuts_sequence(bool bTemporary)
	{	//    
		basic_class key(m_key.get_key_handle(), GetKeyName(bTemporary ? REG_TEMP_KEYS : REG_STATIC_KEYS));
		return key;
	}
	basic_class get_subcatalogs_sequence()
	{	//    
		basic_class key(m_key.get_key_handle(), GetKeyName(REG_SUB_CATALOGS));
		return key;
	}

// //    
public:
	bool set_shortcut(tstring const& name, tstring const& value, bool bTemporary)
	{	/// 
		try {
			CRegistry r(m_key.get_key_handle()
				, GetKeyName(bTemporary ? REG_TEMP_KEYS : REG_STATIC_KEYS));
			r.SetValue(name.c_str(), value.c_str());
		} catch (...) {
			return false;
		}
		return true;
	}
	bool insert_subcatalog(tstring const& name)
	{	// 
		try {
			CRegistry newkey(m_key.get_key_handle(), GetKeyName(REG_SUB_CATALOGS));
			CRegistry hkey(newkey, name.c_str());
		} catch(...)
		{
			return false;
		}
		return true;
	}

//    
	bool delete_shortcut(tstring const& name, bool bTemporary)
	{
		basic_class key(m_key.get_key_handle(), GetKeyName(bTemporary ? REG_TEMP_KEYS : REG_STATIC_KEYS), KEY_ALL_ACCESS);
		return ERROR_SUCCESS == ::RegDeleteValue(key.get_key_handle(), name.c_str());
	}
	bool delete_subcatalog(tstring const& _name)
	{
		tstring name = _name;
		if (*name.begin() == SLASH_CATS_CHAR)
			name.erase(name.begin());

		CRegistry r(m_key.get_key_handle(), GetKeyName(REG_SUB_CATALOGS), true, false);
		return ERROR_SUCCESS == ::RegDeleteKey(r, name.c_str());
	}
//   
	bool get_shortcut_info(tstring const& name, bool bTemporary, tstring &value)
	{
		basic_class key(m_key.get_key_handle(), GetKeyName(bTemporary ? REG_TEMP_KEYS : REG_STATIC_KEYS), KEY_ALL_ACCESS);
		DWORD size = 256;
		DWORD type;
		DWORD ErrorCode;
		std::vector<BYTE> buf(size*sizeof(TCHAR));
		do {
			buf.resize(size);
			ErrorCode = ::RegQueryValueEx(key.get_key_handle(), name.c_str(), 0, &type, reinterpret_cast<BYTE*>(&buf[0]), &size);
		} while (ErrorCode == ERROR_MORE_DATA);

		if (ErrorCode != ERROR_SUCCESS) return false;
		if (type != REG_SZ) return false;

		if (size)
			value.assign(reinterpret_cast<TCHAR*>(&buf[0]), size-1);
		return true;
	}
private:
	tstring get_combined_path(TCHAR const* catalog, CCatalog const *parent = 0)
	{	//       
		tstring name = (parent) ? parent->catalog_path() : _T("");
		if (! name.empty()) name += tstring(SLASH_CATS);
		name += catalog;
		return name;
	}
	static tstring const& get_far_reg_key() { return g_pK->m_S.get_NamedFolders_reg_key();}
	static TCHAR const* GetKeyName(tregs_enum index)
	{
		static TCHAR const* regs[] = {_T("keys"), _T("tempkeys"), _T("Catalogs"), _T("\\Catalogs\\")};
		return regs[static_cast<int>(index)];
	};
};

} //sc
